local addonInfo, privateVars = ...

nkPanel.api = {}
privateVars.plugins = {}

privateVars.events = {}
privateVars.events.update = {}

function nkPanel.test()

  local output = nil
  
  for idx = 1,  #nkPanelSetup.panels, 1 do
    if output ~= nil then 
      output = output .. ","
    else
      output = "" 
    end
    output = output .. nkPanelSetup.panels[idx].sort .. ": " .. nkPanelSetup.panels[idx].name    
  end
  
  print (output)

end

function nkPanel.api.registerPlugin (name, body)

	if privateVars.plugins[name] ~= nil then return end
	
	if body.GetMenu ~= nil then 
    body:GetMenu():SetFontSize(nkPanelSetup.menuFontSize)
    body:GetBody():SetWidth(body:GetMenu():GetWidth() + 4)
    body:GetBody():SetHeight(body:GetMenu():GetHeight() + 4)
	end
		
	privateVars.plugins[name] = body
	
	local found = false
	
	for idx = 1, #nkPanelSetup.panels, 1 do		
		if nkPanelSetup.panels[idx].name == name then
			found = true
			if nkPanelSetup.panels[idx].sort == nil then
			  local sort = privateVars.internal.getSort ()
			  nkPanelSetup.panels[idx].sort = sort
			end
			break
		end
	end
	
	if found == false then
	  local sort = privateVars.internal.getSort ()
	  table.insert(nkPanelSetup.panels, { name = name, show = true, sort = sort })
	end
	
	privateVars.lastRegister = Inspect.Time.Real()

end

function privateVars.internal.comperator(w1,w2)
    if w1.sort == nil then return true end
    if w2.sort == nil then return false end
    if w1.sort < w2.sort then
      return true
    end
  end

function privateVars.internal.getSort ()

  table.sort(nkPanelSetup.panels, privateVars.internal.comperator)
  
  local slot = 0
  
  for idx = 1, #nkPanelSetup.panels, 1 do
    if nkPanelSetup.panels[idx].sort ~= idx then
      slot = idx
      break;
    end
  end
  
  if slot == 0 then slot = #nkPanelSetup.panels + 1 end
  
  return slot
  
end

function nkPanel.api.plugin(name, showThumb, customTitle)

	local body, hideBody, thumb, title
	local menuFontSize = nkPanelSetup.menuFontSize

	local ui = UI.CreateFrame('Frame', name, privateVars.uiElements.panel)
	ui:SetHeight(privateVars.uiElements.panel:GetHeight())
			 	
	ui:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (self)
		if nkPanelSetup.autoHide == true then	
			privateVars.uiElements.panel:SetAlpha(1)
			EnKai.fx.pauseEffect (privateVars.uiElements.panel:GetName() .. '.autohide' )
		end
		
		if body ~= nil then
			privateVars.internal.hideAllBodies() 
			
			if Inspect.System.Secure() == false or ui:GetSecureMode() ~= 'restricted' then
				
				local height, width = body:GetHeight(), body:GetWidth()
				body:ClearAll()
				body:SetWidth(width)
				body:SetHeight(height)
				
				if nkPanelSetup.panelPos == 1 then					
					body:SetPoint("TOPLEFT", ui, "BOTTOMLEFT", 0, 0)	
				else
					body:SetPoint("BOTTOMLEFT", ui, "TOPLEFT", 0, 0)										
				end
							
				body:SetVisible(true)
			end 
		end
	end, name .. ".Mouse.Cursor.In")

	ui:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function (self)
		if nkPanelSetup.autoHide == true then 	
			EnKai.fx.updateTime (privateVars.uiElements.panel:GetName() .. '.autohide' )
		end
		
		if body ~= nil then
			
			local mouse = Inspect.Mouse()
			if mouse.x >= body:GetLeft() and mouse.x < body:GetLeft() + body:GetWidth() and
			   mouse.y >= body:GetTop() and mouse.y < body:GetTop() + body:GetHeight() then
			   return
			end
		
			local func = function () 
				if hideBody ~= false then 
					if Inspect.System.Secure() == false or ui:GetSecureMode() ~= 'restricted' then
						body:SetVisible(false) end
					end 
				end
			privateVars.events.update[name .. '.hideBody'] = { func = func, parent = ui, oneTimeOnly = true, oneTimeDelay = 2 }			
		end
	end, name .. ".Mouse.Cursor.Out")
	
	-- drag and drop
	
	ui:EventAttach(Event.UI.Input.Mouse.Left.Down, function (self)		
		if nkPanelSetup.locked ~= false then return end
		if Inspect.System.Secure() == true then return end
		
		privateVars.internal.initMove ()  
	
		self.leftDown = true
		local mouse = Inspect.Mouse()
		
		self.originalXDiff = mouse.x - self:GetLeft()
		self.originalYDiff = mouse.y - self:GetTop()
		
		local left, top, right, bottom = ui:GetBounds()
		
		if nkPanelSetup.alignPlugins == 'right' then ui:ClearPoint("CENTERRIGHT") else ui:ClearPoint("CENTERLEFT") end
		
		ui:SetPoint("TOPLEFT", UIParent, "TOPLEFT", left, top)
		
		privateVars.tempX = self:GetLeft();
		
	end, name .. ".Left.Down")
	
	ui:EventAttach( Event.UI.Input.Mouse.Cursor.Move, function (self, _, x, y)	
		if self.leftDown ~= true then return end
		ui:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x - self.originalXDiff, y - self.originalYDiff)
	end, name .. ".Cursor.Move")
	
	ui:EventAttach( Event.UI.Input.Mouse.Left.Up, function (self)	
		if self.leftDown ~= true then return end
	  self.leftDown = false
		privateVars.internal:ProcessMove(name, ui)			   
	end, name .. ".Left.Up")
	
	ui:EventAttach( Event.UI.Input.Mouse.Left.Upoutside, function (self)
		if self.leftDown ~= true then return end
		self.leftDown = false
		privateVars.internal:ProcessMove(name, ui)
	end	, name .. ".Left.Upoutside")
	
	-- title thumb
	
	if showThumb == true and customTitle ~= true and nkPanelSetup.showThumb == true then	
		thumb = UI.CreateFrame('Texture', name .. '.thumb', ui)
		thumb:SetPoint("CENTERLEFT", ui, "CENTERLEFT")
		thumb:SetWidth(16)
		thumb:SetHeight(16)		
		thumb:SetMouseMasking('limited')	
	end
	
	function ui:SetThumb(imgType, imgPath)
		if thumb == nil then return end
		thumb:SetTextureAsync(imgType, imgPath)
	end
	
	function ui:ShowThumb(flag)
		if thumb == nil then return end
		thumb:SetVisible(flag)
	end
	
	-- text title
	
	if customTitle ~= true then		
		title = UI.CreateFrame('Text', name .. '.title', ui)
	
		if showThumb == true and nkPanelSetup.showThumb == true then		
			title:SetPoint("CENTERLEFT", thumb, "CENTERRIGHT", 2, 0)
		else
			title:SetPoint("CENTERLEFT", ui, "CENTERLEFT")
		end
		
		title:SetFontSize(nkPanelSetup.fontSize)
		title:SetFontColor(nkPanelSetup.fontColor[1], nkPanelSetup.fontColor[2], nkPanelSetup.fontColor[3], 1)
		
		title:SetMouseMasking('limited')
	end	
	
	function ui:SetTitle(text)
		if title == nil then return end
		if ui:GetSecureMode() == 'restricted' and Inspect.System.Secure() == true then return end
		
		title:ClearWidth()
		title:SetText(text, true)
		if showThumb == true then
			ui:SetWidth(title:GetWidth() + 18)
		else
			ui:SetWidth(title:GetWidth())
		end
	end
	
	function ui:SetTitleFontSize(fontSize)
		if title == nil then return end
		title:SetFontSize(fontSize)
		ui:SetTitle(title:GetText())
	end
	
	function ui:SetTitleColor (r, g, b, a)
		if title == nil then return end
		title:SetFontColor(r, g, b, a)
	end
	
	-- custom title

	function ui:SetCustomTitle(object)
		if customTitle ~= true then return end
		local height, width  = object:GetHeight(), object:GetWidth()
		object:ClearAll()
		object:SetWidth(width)
		object:SetHeight(height)
		object:SetParent(ui)		
		object:SetPoint("CENTERLEFT", ui, "CENTERLEFT")
		object:SetVisible(true)
		object:SetMouseMasking('limited')
		ui:SetWidth(object:GetWidth())
	end

	-- body	
	
	function ui:CreateBody()
		if body == nil then
			body = UI.CreateFrame('Canvas', name .. '.body', ui)
			body:SetVisible(false)
			body:SetPoint("TOPLEFT", ui, "BOTTOMLEFT", 0, 0)
			body:SetWidth(0)
			body:SetHeight(0)
			
			local canvas = {{xProportional = 0, yProportional = 0}, 
							{xProportional = 1, yProportional = 0}, 
							{ xProportional = 1, yProportional = 1}, 
							{ xProportional = 0, yProportional = 1},
							{xProportional = 0, yProportional = 0}}
			local fill = {type = 'solid', r = 0, g = 0, b = 0, a = .8}
			local stroke =  { r = 0.925, g = 0.894, b = 0.741, a = 1, thickness = 1}
			body:SetShape(canvas, fill, stroke)
			
			local oSetVisible = body.SetVisible
			
			function body:SetVisible(flag)
			
				if Inspect.System.Secure() == true then return end
			
				if flag == false then
					privateVars.shownBody = nil
				else
					privateVars.shownBody = body
				end
			
				oSetVisible(self, flag)
				if nkPanelSetup.autoHide == true then	
					EnKai.fx.updateTime (privateVars.uiElements.panel:GetName() .. '.autohide' )
				end
			end
			
			body:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (self)
				hideBody = false
				
				if nkPanelSetup.autoHide == true then
					EnKai.fx.pauseEffect (privateVars.uiElements.panel:GetName() .. '.autohide' )
				end
			end, name .. ".body.Mouse.Cursor.In")
			
			body:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function (self)
			
				local mouse = Inspect.Mouse()

				if mouse.x >= body:GetLeft() and mouse.x < body:GetLeft() + body:GetWidth() and
				   mouse.y >= body:GetTop() and mouse.y < body:GetTop() + body:GetHeight() then
				   return
				end
			
				body:SetVisible(false)
			end, name .. ".body.Mouse.Cursor.Out")
			
		end
		
		return body
	end
	
	function ui:HideBody()
		if body == nil then return end
		body:SetVisible(false)
	end
	
	function ui:SetUpdate(func, oneTimeOnly, oneTimeDelay)
		if func == nil then
			privateVars.events.update[name] = nil
		else		
			privateVars.events.update[name] = { func = func, parent = ui, oneTimeOnly = oneTimeOnly, oneTimeDelay = oneTimeDelay }
		end
	end
	
	function ui:SetMenuFontSize(newFontSize)
	  menuFontSize = newFontSize
	  if ui.GetMenu ~= nil then 
      ui:GetMenu():SetFontSize(newFontSize)
      ui:GetBody():SetWidth(ui:GetMenu():GetWidth() + 4) 
      ui:GetBody():SetHeight(ui:GetMenu():GetHeight() + 4)
	  end
	end
	
	ui:SetVisible(false)
	
	return ui	

end 

function nkPanel.api.preventBodyClose ()

	privateVars.shownBody = nil	

end

function nkPanel.api.setBodyClose (body)

	privateVars.shownBody = body	

end

function nkPanel.api.togglePanel(name, flag)

	for idx = 1, #nkPanelSetup.panels, 1 do
		if nkPanelSetup.panels[idx].name == name then
			nkPanelSetup.panels[idx].show = flag
			break
		end
	end
	privateVars.internal.updatePanel()

end

--------------------------------------------------------------------------------------------------------------------------------------------

function privateVars.internal.initMove ()

  local x = privateVars.plugins [nkPanelSetup.panels[1].name]:GetLeft()
  
	for idx = 1, #nkPanelSetup.panels, 1 do
		if nkPanelSetup.panels[idx] ~= nil then
			if nkPanelSetup.panels[idx].show == true and privateVars.plugins [nkPanelSetup.panels[idx].name] ~= nil then
				local plugin = privateVars.plugins [nkPanelSetup.panels[idx].name]
				
				if nkPanelSetup.alignPlugins == 'right' then
				  plugin:ClearPoint("CENTERRIGHT")
				else
				  plugin:ClearPoint("CENTERLEFT")
        end
        
        plugin:SetPoint("CENTERLEFT", privateVars.uiElements.panel, "CENTERLEFT", x, 0)
				x = x + plugin:GetWidth() + nkPanelSetup.spacing
			end				
		end
	end

end

function privateVars.internal.ProcessMove (_, movedPanelName, movedPanelObject)

  local diff = 0

  if privateVars.tempX < movedPanelObject:GetLeft() then
    diff = -1
  end

	local thisPos, newPos = 0, 0
	
  if movedPanelObject:GetLeft() < privateVars.plugins[nkPanelSetup.panels[1].name]:GetLeft() then
    newPos = 1
  else
    local lastNo
    for idx = 1, #nkPanelSetup.panels, 1 do
      if nkPanelSetup.panels[idx].sort == 9999 then
        lastNo = idx -1
        break;
      end
    end
    
    if lastNo ~= nil then
       if movedPanelObject:GetLeft() > privateVars.plugins[nkPanelSetup.panels[lastNo].name]:GetLeft() then
         newPos = lastNo + 1
       end
    end
  end
	
	for idx = 1, #nkPanelSetup.panels, 1 do
		local checkName = privateVars.plugins[nkPanelSetup.panels[idx].name]:GetName()
	
		if checkName == movedPanelName then thisPos = idx end
	  
		if newPos == 0 and nkPanelSetup.panels[idx].show == true then
			local checkPlugin = privateVars.plugins[nkPanelSetup.panels[idx].name]
			if checkPlugin ~= nil then
			  local thisLeft = checkPlugin:GetLeft()
			  if thisPos == idx then thisLeft = privateVars.tempX end
			
        if thisLeft >= movedPanelObject:GetLeft() then
          newPos = idx + diff
				end
			end		
		end
	end
	
	--print (thisPos, newPos)
	
	if thisPos ~= 0 and newPos ~= 0 then
	  	
		local realName = nkPanelSetup.panels[thisPos].name
		
		if newPos ~= thisPos then
		
  		if newPos > thisPos then
  		  for idx = thisPos +1, newPos, 1 do
  		    --print (nkPanelSetup.panels[idx].name .. "=> " .. (nkPanelSetup.panels[idx].sort + 1))
  		    nkPanelSetup.panels[idx].sort = nkPanelSetup.panels[idx].sort - 1
  		  end
  		else
  		  for idx = newPos, thisPos-1, 1 do
        --print (nkPanelSetup.panels[idx].name .. "=> " .. (nkPanelSetup.panels[idx].sort + 1))
          nkPanelSetup.panels[idx].sort = nkPanelSetup.panels[idx].sort + 1  
        end
  		end
  		
  		nkPanelSetup.panels[thisPos].sort = newPos
  		
  		table.sort(nkPanelSetup.panels, privateVars.internal.comperator)
  		
  		--if thisPos < newPos then newPos = newPos -1 end		
  
  		--table.remove(nkPanelSetup.panels, thisPos, 1)
  		
  		--if newPos == 0 then
  		--	table.insert(nkPanelSetup.panels, {name = realName, show = true})
  		--else
  		--	table.insert(nkPanelSetup.panels, newPos, {name = realName, show = true})
  		--end
  	end
  end
	
	privateVars.internal.updatePanel()		

end

function privateVars.internal.updatePanel ()

	local x, object, from, to = 40, privateVars.uiElements.panel, "CENTERLEFT", "CENTERLEFT"
  local start, stop, increment = 1, #nkPanelSetup.panels, 1
  
	if nkPanelSetup.alignPlugins == 'right' then 
	  x, from, to = -10, "CENTERRIGHT", "CENTERRIGHT"
	  start, stop, increment = #nkPanelSetup.panels, 1, -1
	end
	
	local pluginWidth = 0
	
	table.sort(nkPanelSetup.panels, privateVars.internal.comperator)

	for idx = start, stop, increment do
		if nkPanelSetup.panels[idx] ~= nil then
			if privateVars.plugins[nkPanelSetup.panels[idx].name] ~= nil then
							
				local panel = privateVars.plugins [nkPanelSetup.panels[idx].name]
				
				if nkPanelSetup.panels[idx].show == false then
					panel:SetVisible(false)
				else						
					panel:ShowThumb(nkPanelSetup.showThumb)		
					local width, height = panel:GetWidth(), panel:GetHeight()
					panel:ClearAll()
					panel:SetWidth(width)
					panel:SetHeight(height)
					panel:SetPoint(from, object, to, x, 0)
					panel:SetVisible(true)		
					x, object, to = nkPanelSetup.spacing, panel, "CENTERRIGHT"
					if nkPanelSetup.alignPlugins == 'right' then 
					  to = "CENTERLEFT"
					  x = x * -1 
					end
					
					if nkPanelSetup.alignPlugins == 'center' then
					  pluginWidth = pluginWidth + width
					  if idx > 1 then pluginWidth = pluginWidth + nkPanelSetup.spacing end
					end 
					
				end
			else
				table.remove(nkPanelSetup.panels, idx, 1)
			end	
		end
	end
	
	if nkPanelSetup.alignPlugins == 'center' then
	  privateVars.plugins [nkPanelSetup.panels[1].name]:ClearPoint("CENTERLEFT")
	  
	  local startX = (privateVars.uiElements.panel:GetWidth() - pluginWidth) / 2
	  privateVars.plugins [nkPanelSetup.panels[1].name]:SetPoint("CENTERLEFT", privateVars.uiElements.panel, "CENTERLEFT", startX, 0 )
	end

end

function privateVars.internal.updateTitles ()

	for name, panel in pairs(privateVars.plugins) do
		panel:SetTitleFontSize(nkPanelSetup.fontSize)
		panel:SetTitleColor(nkPanelSetup.fontColor[1], nkPanelSetup.fontColor[2], nkPanelSetup.fontColor[3], 1)
	end
		
	privateVars.internal.updatePanel()

end

function privateVars.internal.updateMenus()

  for name, panel in pairs(privateVars.plugins) do
    panel:SetMenuFontSize(nkPanelSetup.menuFontSize)
  end

end

function privateVars.internal.hideAllBodies()

	local secureMode = Inspect.System.Secure()

	for k, v in pairs (privateVars.plugins) do
		if secureMode == false or v:GetSecureMode() ~= 'restricted' then 
			v:HideBody()
		end
	end


end

function privateVars.internal.updateEvent ()

	local doUpdate = false

	if privateVars.lastPanelUpdate == nil then
		privateVars.lastPanelUpdate = Inspect.Time.Real()
		doUpdate = true
	else
		if EnKai.tools.math.round((Inspect.Time.Real() - privateVars.lastPanelUpdate), 1) > 1 then doUpdate = true end
	end
		
	if doUpdate == true then 
		privateVars.lastPanelUpdate = Inspect.Time.Real()
		
		if privateVars.lastRegister ~= nil and EnKai.tools.math.round((Inspect.Time.Real() - privateVars.lastRegister), 1) > 2 then
		  if Inspect.System.Secure() ~= true then		
        privateVars.lastRegister = nil
        privateVars.internal.updatePanel ()
        privateVars.uiElements.panel:SetVisible(true)
      end
		end

		if privateVars.shownBody ~= nil then
			local mouse = Inspect.Mouse()
			
			if mouse.x < privateVars.shownBody:GetLeft() or mouse.x > privateVars.shownBody:GetRight() then
				privateVars.shownBody:SetVisible(false)
			elseif nkPanelSetup.panelPos == 1 then
				if mouse.y < privateVars.shownBody:GetParent():GetTop() or mouse.y > privateVars.shownBody:GetBottom() then
					privateVars.shownBody:SetVisible(false)
				end
			else
				if mouse.y < privateVars.shownBody:GetTop() or mouse.y > privateVars.shownBody:GetParent():GetBottom() then
					privateVars.shownBody:SetVisible(false)
				end
			end 
		end

		for k, v in pairs(privateVars.events.update) do

			if v.oneTimeOnly == true and v.oneTimeDelay ~= nil then
				v.oneTimeDelay = v.oneTimeDelay - 1
				if v.oneTimeDelay <= 0 then 
					v.oneTimeDelay = nil
					v.func(v.parent) 
				end 
			else
				v.func(v.parent)
			end 
			
			if v.oneTimeOnly == true and v.oneTimeDelay == nil then privateVars.events.update[k] = nil end
		end
	end

end

Command.Event.Attach(Event.System.Update.Begin, privateVars.internal.updateEvent, "nkPanel.System.Update.Begin")