local addonInfo, privateVars = ...

function privateVars.ui.configTabPlugins (parent)

	local name = 'privateVars.ui.config.TabPane.TabPlugins'

	local tabPane = UI.CreateFrame ("Frame", name, parent)
	local grid, slider, label
	
	local gridRows = 17
	
	function tabPane:build ()
	
	  label = UI.CreateFrame("Text", name .. '.label', tabPane)
    label:SetPoint("TOPLEFT", tabPane, "TOPLEFT", 0, 0)
    label:SetText(privateVars.langTexts.labelPlugins)
    label:SetFontSize(16)
    label:SetFontColor(1, 1, 1, 1) 
	
		-- grid definition
		
		grid = EnKai.uiCreateFrame("nkGrid", name .. '.grid', tabPane)
		
		grid:SetHeaderHeight(0)
		grid:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 0, 10)
		grid:SetBorderColor(0.086, 0.235, 0.392, 1)
		grid:SetBodyColor(0.153, 0.314, 0.490, 1)
		grid:SetTransparentHeader()
		--grid:SetSelectable(true)
		grid:SetLayer(2)
		
		local cols = {	{ align = 'center', texture = true, textureType = 'nkPanel', textureHeight = 14, textureWidth = 14, width = 20,  }, 						
						        { align = 'left', width = 390 } }	
		
		grid:SetVisible(false)	
		grid:Layout (cols, gridRows)	
									
		Command.Event.Attach(EnKai.events[name .. '.grid'].GridFinished, function ()
			tabPane:PopulateGrid() 
			grid:SetVisible(true)
		end, name .. '.grid.GridFinished')
		
		Command.Event.Attach(EnKai.events[name .. '.grid'].WheelForward, function (_, rowPos)			
			slider:AdjustValue (rowPos)
		end, name .. '.grid.Grid.WheelForward')
		
		Command.Event.Attach(EnKai.events[name .. '.grid'].WheelBack, function (_, rowPos)
			slider:AdjustValue (rowPos)
		end, name .. '.grid.Grid.WheelBack')
		
		Command.Event.Attach(EnKai.events[name .. '.grid'].LeftClick, function (_, row)
			
			if grid:GetKey(row) == nil then return end
			
			local cellValue = grid:GetCellValue(row, 1)
			local active = false
			
			if cellValue == 'gfx/statusRed.png' then
				grid:SetCellValue(row, 1, 'gfx/statusGreen.png')
				active = true
			else
				grid:SetCellValue(row, 1, 'gfx/statusRed.png')
				active = false
			end
			
			for idx = 1, #nkPanelSetup.panels, 1 do
				if nkPanelSetup.panels[idx].name == grid:GetKey(row) then
					nkPanelSetup.panels[idx].show = active
					break
				end
			end
			privateVars.internal.updatePanel()
			
		end, name .. '.grid.LeftClick')				
		
		-- grid slider
		
		slider = EnKai.uiCreateFrame("nkScrollbox", name .. ".slider", tabPane)	
		
		slider:SetPoint("TOPLEFT", grid, "TOPRIGHT")
		slider:SetHeight(grid:GetHeight())
		slider:SetRange(1, 1)
		slider:SetVisible(false)
		slider:SetLayer(2)
		slider:SetColor(1, 1, 1, 1)
		
		Command.Event.Attach(EnKai.events[name .. ".slider"].ScrollboxChanged, function ()		
			grid:SetRowPos(math.floor(slider:GetValue('value')), true)
		end, name .. '.slider.ScrollboxChanged')
		
	end
																
	function tabPane:PopulateGrid ()
	
		local gridValues = {}
		
		for name, id in pairs (privateVars.plugins) do

			for idx = 1, #nkPanelSetup.panels, 1 do
				if nkPanelSetup.panels[idx].name == name then
					local status = 'gfx/statusRed.png'
					if nkPanelSetup.panels[idx].show == true then status = 'gfx/statusGreen.png' end
					
					table.insert (gridValues, {{ key = name, value = status }, { value = name} })
					break 
				end
			end
		end
		
		table.sort (gridValues, function (v1, v2) return string.upper(v1[2].value) < string.upper(v2[2].value) end )
		
		grid:SetRowPos(1, false)
		grid:SetCellValues(gridValues)

		if (#gridValues-gridRows) > 0 then		
			slider:SetRange (1, #gridValues-gridRows+1)
			slider:AdjustValue(1)
			slider:SetVisible(true)			
		else
			slider:SetVisible(false)
		end
		
	end
	
	return tabPane
	
end
