local addonInfo, privateVars = ...

function privateVars.ui.configTabSettings (parent)

	local name = 'privateVars.ui.config.TabPane.TabSettings'

	local tabPane = UI.CreateFrame ("Frame", name, parent)
	local labelPanel, showRadio, posRadio, autoHideTimerSlider, panelColor, panelAlphaSlider, alignRadio, textureSelect
	local lockedCheckbox, spacingSlider, labelPlugins, fontSizeSlider, fontColor, showThumbCheckbox, menuSizeSlider

	function tabPane:build ()
	
		labelPanel = EnKai.uiCreateFrame("nkText", name .. '.labelPanel', tabPane)
		labelPanel:SetPoint("TOPLEFT", tabPane, "TOPLEFT")
		labelPanel:SetText(privateVars.langTexts.labelPanelSettings)
		labelPanel:SetFontSize(16)
			
		showRadio = EnKai.uiCreateFrame("nkRadioButton", name .. '.showRadio', tabPane)	
		showRadio:SetPoint("TOPLEFT", labelPanel, "BOTTOMLEFT", 0, 10)
		showRadio:SetWidth(300)
		showRadio:SetLabelWidth(190)
		showRadio:SetText(privateVars.langTexts.showPanelRadio)
		showRadio:SetSelection({{ value = false, label = privateVars.langTexts.always}, { value = true, label = privateVars.langTexts.autohide}})
		showRadio:SetSelectedValue(nkPanelSetup.autoHide)
	
		Command.Event.Attach(EnKai.events[name .. '.showRadio'].RadiobuttonChanged, function (_, newValue)
			nkPanelSetup.autoHide = newValue
			privateVars.uiElements.panel:SetAutoHide(newValue)
		end, name .. '.showRadio.RadiobuttonChanged')
		
		posRadio = EnKai.uiCreateFrame("nkRadioButton", name .. '.posRadio', tabPane)	
		posRadio:SetPoint("TOPLEFT", showRadio, "BOTTOMLEFT", 0, 10)
		posRadio:SetWidth(300)
		posRadio:SetLabelWidth(190)
		posRadio:SetText(privateVars.langTexts.posPanelRadio)
		posRadio:SetSelection({{ value = 1, label = privateVars.langTexts.top}, { value = 2, label = privateVars.langTexts.bottom}})
		posRadio:SetSelectedValue(nkPanelSetup.panelPos)
	
		Command.Event.Attach(EnKai.events[name .. '.posRadio'].RadiobuttonChanged, function (_, newValue)
			local changeFunc = function()
				nkPanelSetup.panelPos = newValue
				if nkPanelSetup.panelPos == 1 then
					privateVars.uiElements.panel:ClearPoint("BOTTOMLEFT")
					privateVars.uiElements.panel:ClearPoint("BOTTOMRIGHT")
					privateVars.uiElements.panel:SetPoint("TOPLEFT", UIParent, "TOPLEFT")	
					privateVars.uiElements.panel:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT")
				else
					privateVars.uiElements.panel:ClearPoint("TOPLEFT")
					privateVars.uiElements.panel:ClearPoint("TOPRIGHT")
					privateVars.uiElements.panel:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT")	
					privateVars.uiElements.panel:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT")
				end
			end
			
			EnKai.events.addInsecure(changeFunc)
		end, name .. '.posRadio.RadiobuttonChanged')
		
		autoHideTimerSlider = EnKai.uiCreateFrame("nkSlider", name .. '.autoHideTimerSlider', tabPane)	
		autoHideTimerSlider:SetPoint("TOPLEFT", posRadio, "BOTTOMLEFT", 0, 10)
		autoHideTimerSlider:SetWidth(300)
		autoHideTimerSlider:SetRange(1, 60)
		autoHideTimerSlider:SetLabelWidth(190)
		autoHideTimerSlider:SetText(string.format(privateVars.langTexts.autoHideTimerSlider, nkPanelSetup.autoHideTimer))
		autoHideTimerSlider:SetPrecision(1)
		autoHideTimerSlider:AdjustValue(nkPanelSetup.autoHideTimer)
    		
		Command.Event.Attach(EnKai.events[name .. '.autoHideTimerSlider'].SliderChanged, function (_, newValue)
			nkPanelSetup.autoHideTimer = newValue
			autoHideTimerSlider:SetText(string.format(privateVars.langTexts.autoHideTimerSlider, nkPanelSetup.autoHideTimer))
		end, name .. '.spacingSlider.autoHideTimerSlider')
				
		panelColor = EnKai.uiCreateFrame("nkColorPicker", name .. '.panelColorPicker', tabPane)
    panelColor:SetPoint("TOPLEFT", autoHideTimerSlider, "BOTTOMLEFT", 0, 7)
    panelColor:SetLayer(10)
    panelColor:SetWidth(205)
    panelColor:SetHeight(15)
    panelColor:SetColor(nkPanelSetup.panelColor[1], nkPanelSetup.panelColor[2], nkPanelSetup.panelColor[3], 1)
    panelColor:SetText(privateVars.langTexts.panelColor)
    
    Command.Event.Attach(EnKai.events[name .. '.panelColorPicker'].ColorChanged, function (_, r, g, b, a)   
      nkPanelSetup.panelColor = { r, g, b}
      privateVars.uiElements.panel:SetBackgroundColor(nkPanelSetup.panelColor[1], nkPanelSetup.panelColor[2], nkPanelSetup.panelColor[3], nkPanelSetup.panelAlpha)        
    end, name .. ".panelColorPicker.ColorChanged")
		
		local selValues = {}
    table.insert(selValues, { label = privateVars.langTexts.none, value = '', texturePath = '', textureType = '' })
    
    for k, v in pairs (privateVars.internal.textureList()) do
      table.insert (selValues, { label = "Texture " .. k, value = k, texturePath = v, textureType = 'EnKai' })
    end
				
		textureSelect = EnKai.uiCreateFrame("nkCombobox", name .. '.textureSelect', tabPane)
    textureSelect:SetPoint("TOPLEFT", panelColor, "BOTTOMLEFT", 0, 7)
    textureSelect:SetWidth(300)
    textureSelect:SetLabelWidth(190)
    textureSelect:SetText(privateVars.langTexts.textureSelect)
    textureSelect:SetSelection(selValues)
    textureSelect:SetSelectedValue(nkPanelSetup.panelTexture, false)
    textureSelect:SetLayer(9)
    
    Command.Event.Attach(EnKai.events[name .. '.textureSelect'].ComboChanged, function (_, newValue)
      nkPanelSetup.panelTexture = newValue.value
      privateVars.uiElements.panel:UpdateTexture()       
    end, name .. ".textureSelect.ColorChanged")
				
		panelAlphaSlider = EnKai.uiCreateFrame("nkSlider", name .. '.panelAlphaSlider', tabPane)	
		panelAlphaSlider:SetPoint("TOPLEFT", textureSelect, "BOTTOMLEFT", 0, 10)
		panelAlphaSlider:SetWidth(300)
		panelAlphaSlider:SetRange(0, 100)
		panelAlphaSlider:SetLabelWidth(190)
		panelAlphaSlider:SetText(privateVars.langTexts.panelAlphaSlider)
		panelAlphaSlider:SetPrecision(1)
		panelAlphaSlider:AdjustValue(nkPanelSetup.panelAlpha * 100)
		
		Command.Event.Attach(EnKai.events[name .. '.panelAlphaSlider'].SliderChanged, function (_, newValue)
			nkPanelSetup.panelAlpha = newValue / 100
			privateVars.uiElements.panel:SetBackgroundColor(nkPanelSetup.panelColor[1], nkPanelSetup.panelColor[2], nkPanelSetup.panelColor[3], nkPanelSetup.panelAlpha)	
		end, name .. '.panelAlphaSlider.SliderChanged')
			
		labelPlugins = EnKai.uiCreateFrame("nkText", name .. '.labelPlugins', tabPane)
		labelPlugins:SetPoint("TOPLEFT", panelAlphaSlider, "BOTTOMLEFT", 0, 25)
		labelPlugins:SetEffectGlow({ offsetX = 2, offsetY = 2})
		labelPlugins:SetText(privateVars.langTexts.labelPluginsSettings)
		labelPlugins:SetFontSize(16)
		
		lockedCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. '.lockedCheckbox', tabPane)	
		lockedCheckbox:SetPoint("TOPLEFT", labelPlugins, "BOTTOMLEFT", 0, 10)
		lockedCheckbox:SetLabelWidth(185)
		lockedCheckbox:SetText(privateVars.langTexts.lockedCheckbox)
		lockedCheckbox:SetChecked(nkPanelSetup.locked)
		lockedCheckbox:SetLabelInFront(true)
		
		Command.Event.Attach(EnKai.events[name .. '.lockedCheckbox'].CheckboxChanged, function (_, newValue)		
			nkPanelSetup.locked = newValue 
		end, name .. '.lockedCheckbox.CheckboxChanged')
		
		alignRadio = EnKai.uiCreateFrame("nkRadioButton", name .. '.alignRadio', tabPane) 
    alignRadio:SetPoint("TOPLEFT", lockedCheckbox, "BOTTOMLEFT", 0, 10)
    alignRadio:SetWidth(300)
    alignRadio:SetLabelWidth(190)
    alignRadio:SetText(privateVars.langTexts.alignRadio)
    alignRadio:SetSelection({{ value = 'left', label = privateVars.langTexts.left}, { value = 'center', label = privateVars.langTexts.center}, { value = 'right', label = privateVars.langTexts.right}})
    alignRadio:SetSelectedValue(nkPanelSetup.alignPlugins)
  
    Command.Event.Attach(EnKai.events[name .. '.alignRadio'].RadiobuttonChanged, function (_, newValue)
      nkPanelSetup.alignPlugins = newValue
      privateVars.internal.updatePanel ()
    end, name .. '.alignRadio.RadiobuttonChanged')
		
		spacingSlider = EnKai.uiCreateFrame("nkSlider", name .. '.spacingSlider', tabPane)	
		spacingSlider:SetPoint("TOPLEFT", alignRadio, "BOTTOMLEFT", 0, 10)
		spacingSlider:SetWidth(300)
		spacingSlider:SetRange(1, 50)
		spacingSlider:SetLabelWidth(190)
		spacingSlider:SetText(string.format(privateVars.langTexts.spacingSlider, nkPanelSetup.spacing))
		spacingSlider:SetPrecision(1)
		spacingSlider:AdjustValue(nkPanelSetup.spacing)
		
		Command.Event.Attach(EnKai.events[name .. '.spacingSlider'].SliderChanged, function (_, newValue)
			nkPanelSetup.spacing = newValue
			spacingSlider:SetText(string.format(privateVars.langTexts.spacingSlider, nkPanelSetup.spacing))
			privateVars.internal.updatePanel ()
		end, name .. '.spacingSlider.SliderChanged')
		
		showThumbCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. '.showThumbCheckbox', tabPane)	
		showThumbCheckbox:SetPoint("TOPLEFT", spacingSlider, "BOTTOMLEFT", 0, 10)
		showThumbCheckbox:SetLabelWidth(185)
		showThumbCheckbox:SetText(privateVars.langTexts.showThumbCheckbox)
		showThumbCheckbox:SetChecked(nkPanelSetup.showThumb)
		showThumbCheckbox:SetLabelInFront(true)
		
		Command.Event.Attach(EnKai.events[name .. '.showThumbCheckbox'].CheckboxChanged, function (_, newValue)		
			nkPanelSetup.showThumb = newValue
			privateVars.internal.updatePanel ()	
		end, name .. '.showThumbCheckbox.CheckboxChanged')
		
		fontSizeSlider = EnKai.uiCreateFrame("nkSlider", name .. '.fontSizeSlider', tabPane)	
		fontSizeSlider:SetPoint("TOPLEFT", showThumbCheckbox, "BOTTOMLEFT", 0, 10)
		fontSizeSlider:SetWidth(300)
		fontSizeSlider:SetRange(8, 30)
		fontSizeSlider:SetLabelWidth(190)
		fontSizeSlider:SetText(string.format(privateVars.langTexts.fontSizeSlider, nkPanelSetup.fontSize))
		fontSizeSlider:SetPrecision(1)
		fontSizeSlider:AdjustValue(nkPanelSetup.fontSize)
		
		Command.Event.Attach(EnKai.events[name .. '.fontSizeSlider'].SliderChanged, function (_, newValue)
			nkPanelSetup.fontSize = newValue
			fontSizeSlider:SetText(string.format(privateVars.langTexts.fontSizeSlider, nkPanelSetup.fontSize))
			privateVars.internal.updateTitles ()
		end, name .. '.fontSizeSlider.SliderChanged')
		
		fontColor = EnKai.uiCreateFrame("nkColorPicker", name .. '.fontColorPicker', tabPane)
		fontColor:SetPoint("TOPLEFT", fontSizeSlider, "BOTTOMLEFT", 0, 7)
		fontColor:SetLayer(8)
		fontColor:SetWidth(205)
		fontColor:SetHeight(15)
		fontColor:SetColor(nkPanelSetup.fontColor[1], nkPanelSetup.fontColor[2], nkPanelSetup.fontColor[3], 1)		
		fontColor:SetText(privateVars.langTexts.fontColor)
		
		Command.Event.Attach(EnKai.events[name .. '.fontColorPicker'].ColorChanged, function (_, r, g, b, a)		
			nkPanelSetup.fontColor = { r, g, b}
			privateVars.internal.updateTitles ()				
		end, name .. ".fontColorPicker.ColorChanged")
	
	  menuSizeSlider = EnKai.uiCreateFrame("nkSlider", name .. '.menuSizeSlider', tabPane)  
    menuSizeSlider:SetPoint("TOPLEFT", fontColor, "BOTTOMLEFT", 0, 10)
    menuSizeSlider:SetWidth(300)
    menuSizeSlider:SetRange(8, 30)
    menuSizeSlider:SetLabelWidth(190)
    menuSizeSlider:SetText(string.format(privateVars.langTexts.menuSizeSlider, nkPanelSetup.menuFontSize))
    menuSizeSlider:SetPrecision(1)
    menuSizeSlider:AdjustValue(nkPanelSetup.menuFontSize)
    
    Command.Event.Attach(EnKai.events[name .. '.menuSizeSlider'].SliderChanged, function (_, newValue)
      nkPanelSetup.menuFontSize = newValue
      menuSizeSlider:SetText(string.format(privateVars.langTexts.menuSizeSlider, nkPanelSetup.menuFontSize))
      privateVars.internal.updateMenus ()
    end, name .. '.menuSizeSlider.SliderChanged')
	
	end
	
	return tabPane
	
end