local addonInfo, privateVars = ...

if nkPanel ~= nil then return end
nkPanel = {}

privateVars.uiElements = {}
privateVars.uiElements.context = UI.CreateContext("nkPanel")
privateVars.uiElements.context:SetStrata ('topmost')
privateVars.uiElements.context:SetSecureMode('restricted')

privateVars.uiElements.contextConfig = UI.CreateContext("privateVars.ui.config")
privateVars.uiElements.contextConfig:SetStrata ('dialog')

privateVars.uiElements.contextDialog = UI.CreateContext("nkPanel.Dialog")
privateVars.uiElements.contextDialog:SetStrata ('topmost')

privateVars.internal = {}
privateVars.ui = {}

function nkPanel.main(_, addon)

	if addon == addonInfo.identifier then
	
		privateVars.uiElements.panel = nkPanel.buildPanel()

		nkPanel.api.registerPlugin('default: clock plugin', nkPanel.plugins.clock())
		nkPanel.api.registerPlugin('default: coords plugin', nkPanel.plugins.coords())
		nkPanel.api.registerPlugin('default: cpu usage plugin', nkPanel.plugins.cpu())
		nkPanel.api.registerPlugin('default: bag count plugin', nkPanel.plugins.bags())
		nkPanel.api.registerPlugin('default: experience tracker plugin', nkPanel.plugins.experience())
		nkPanel.api.registerPlugin('default: fps plugin', nkPanel.plugins.fps())
		nkPanel.api.registerPlugin('default: level plugin', nkPanel.plugins.level())
		nkPanel.api.registerPlugin('default: location tracker plugin', nkPanel.plugins.location())
		nkPanel.api.registerPlugin('default: social plugin', nkPanel.plugins.social())
		nkPanel.api.registerPlugin('default: money plugin', nkPanel.plugins.platinium())
		nkPanel.api.registerPlugin('default: shard plugin', nkPanel.plugins.shard())
		nkPanel.api.registerPlugin('default: reloadui plugin', nkPanel.plugins.reloadui())
	end
	
end

function nkPanel.settingsHandler(_, addon) 

	if addon == addonInfo.identifier then
	
		if nkPanelSetup == nil then
			nkPanelSetup = { panelPos = 1, showThumb = true, autoHide = false, autoHideTimer = 10, locked = false, spacing = 10, fontSize = 13, menuSizeSlider = 13, fontColor = {1,1,1}, panelColor = {0,0,0}, panelAlpha = .6, panels = {}}
		end
		
		-- 1.0.5 settings change
		
		if nkPanelSetup.showThumb == nil then nkPanelSetup.showThumb = true end
		
		-- 1.1.0 settings change
		
		if nkPanelSetup.panelPos == nil then nkPanelSetup.panelPos = 1 end
		
		-- 1.2.0 settings change
		
		if type(nkPanelSetup.panelColor) ~= 'table' then
			local color = { black = { 0, 0, 0 }, white = {1, 1, 1}, red = { 1, 0, 0}, yellow = {1, 0.85, 0}, blue = { 0, 0, 1}}			
			nkPanelSetup.panelColor = color[nkPanelSetup.panelColor]
		end		
		
		if type(nkPanelSetup.fontColor) ~= 'table' then
			local color = { black = { 0, 0, 0 }, white = {1, 1, 1}, red = { 1, 0, 0}, yellow = {1, 0.85, 0}, blue = { 0, 0, 1}}			
			nkPanelSetup.fontColor = color[nkPanelSetup.fontColor]
		end
		
		-- 1.3.0 settings change
		
		if nkPanelSetup.panels ~= nil and #nkPanelSetup.panels > 0 and nkPanelSetup.panels[1].sort == nil then		
  		for idx = 1, #nkPanelSetup.panels, 1  do
  		  nkPanelSetup.panels[idx].sort = idx
  		end
    end
    
    -- 1.4.2 settings change
    
    if nkPanelSetup.alignPlugins == nil then nkPanelSetup.alignPlugins = 'left' end
    if nkPanelSetup.panelTexture == nil then nkPanelSetup.panelTexture = '' end
    
    -- make sure that the sort is always proper on addon load
    
    if nkPanelSetup.menuFontSize == nil then nkPanelSetup.menuFontSize = 13 end
    
    table.sort(nkPanelSetup.panels, privateVars.internal.comperator)
    
    local count = 1
    
    for idx = 1, #nkPanelSetup.panels, 1 do
      if nkPanelSetup.panels[idx].show == true then      
        nkPanelSetup.panels[idx].sort = count
        count = count + 1
      else
        nkPanelSetup.panels[idx].sort = 9999
      end
    end
    
    table.sort(nkPanelSetup.panels, privateVars.internal.comperator)
		
		
	end
					
end

Command.Event.Attach(Event.Addon.Load.End, nkPanel.main, "nkPanel.Addon.Load.End")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, nkPanel.settingsHandler, "nkPanel.SavedVariables.Load.End")

--------------------------------------------------------------------------------------------------------------------------------------------


function nkPanel.buildPanel ()
	
	local name = 'nkPanel'

	local color = { black = { 0, 0, 0 }, white = {1, 1, 1}, red = { 1, 0, 0}, yellow = {1, 0.85, 0}, blue = { 0, 0, 1}}		

	local ui = UI.CreateFrame('Frame', name, privateVars.uiElements.context)
	local texture = EnKai.uiCreateFrame("nkTexture", name .. ".background", privateVars.uiElements.context)
	
	ui:SetVisible(false)
	texture:SetVisible(false)
	
	if nkPanelSetup.panelPos == 1 then
		ui:SetPoint("TOPLEFT", UIParent, "TOPLEFT")	
		ui:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT")
	else
		ui:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT")	
		ui:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT")
	end
	
	ui:SetHeight(20)
	ui:SetBackgroundColor(nkPanelSetup.panelColor[1], nkPanelSetup.panelColor[2], nkPanelSetup.panelColor[3], nkPanelSetup.panelAlpha)
	ui:SetSecureMode('restricted')
	ui:SetLayer(2)
	
	texture:SetPoint("TOPLEFT", ui, "TOPLEFT")
	texture:SetPoint("BOTTOMRIGHT", ui, "BOTTOMRIGHT")
	texture:SetLayer(1)
	
	local img = UI.CreateFrame('Texture', name .. '.img', ui)
	img:SetPoint("CENTERLEFT", ui, "CENTERLEFT", 2, 0)
	img:SetTextureAsync('nkPanel', 'gfx/nk.png')
	
	img:EventAttach(Event.UI.Input.Mouse.Left.Down, function (self)
		if privateVars.uiElements.config == nil then privateVars.uiElements.config = privateVars.ui.config() end
		privateVars.uiElements.config:SetVisible(true)
	end, name .. '.Mouse.Left.Down')
	
	local oSetAlpha, oSetVisible = ui.SetAlpha, ui.SetVisible
  
  function ui:SetAlpha(newAlpha)   
    oSetAlpha(self, newAlpha)
    texture:SetAlpha(newAlpha)
  end
  
  function ui:SetVisible(newFlag)
    oSetVisible(self, newFlag)
    if nkPanelSetup.panelTexture ~= '' then texture:SetVisible(newFlag) end
  end
	
	function ui:UpdateTexture()
  	if nkPanelSetup.panelTexture ~= '' then
      texture:SetTextureAsync("nkPanel", privateVars.internal.getTextureById(nkPanelSetup.panelTexture))
    else
      texture:SetVisible(false)
    end
	end
	
	function ui:SetAutoHide(flag)
	
		if flag == true then
	
			ui:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (self)		
				ui:SetAlpha(1)
				EnKai.fx.pauseEffect (name .. '.autohide' )
			end, name .. ".Mouse.Cursor.In")
		
			ui:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function (self)
				EnKai.fx.updateTime (name .. '.autohide' )
			end, name .. ".Mouse.Cursor.Out")
			
			local func = function ()
				if Inspect.System.Secure() == true then
					EnKai.fx.updateTime (name .. '.autohide' )
				else
					ui:SetAlpha(0)
				end
			end
			
			EnKai.fx.register (name .. '.autohide', ui, { id = 'timedhide', duration = 5, callback = func  } )
		else
			
			ui:EventDetach(Event.UI.Input.Mouse.Cursor.In, nil, name .. ".Mouse.Cursor.In")
			ui:EventDetach(Event.UI.Input.Mouse.Cursor.Out, nil, name .. ".Mouse.Cursor.Out")
			
			EnKai.fx.cancel(name .. '.autohide')
			ui:SetAlpha(1)
			
		end
		
	end
	
	if nkPanelSetup.autoHide == true then ui:SetAutoHide(true) end
	ui:UpdateTexture()
		
	return ui

end

function nkPanel.messageDialog (message)

	if privateVars.uiElements.messageDialog == nil then
		privateVars.uiElements.messageDialog = EnKai.uiCreateFrame("nkDialog", "nkPanel.messageDialog", privateVars.uiElements.contextDialog)
		privateVars.uiElements.messageDialog:SetWidth(500)
		privateVars.uiElements.messageDialog:SetHeight(200)
		privateVars.uiElements.messageDialog:SetType('message')
	end
	
	privateVars.uiElements.messageDialog:SetMessage(message)
	privateVars.uiElements.messageDialog:SetVisible(true)

end